# Clase 05-04-24

\tableofcontents

## Formatos Soportados por Three.js

* glTF
* FBX
* OBJ
* COLLADA (DAE)
* STL

Siendo el más importante y eficiente `glTF`

* Dos versiones 1.0 y 2.0, siendo la primera obsoleta

## GLTF

Los archivos `.gltf` son modelos 3D en formato JSON.

Los archivos `.glb` son la version binaria de los archivos `gltf`

## Loaders

```js
import { GLTFLoader } from 'three/examples/jsm/loaders/GLTFLoader';
```

```js
loader.load( 'models/helmet/DamagedHelmet.gltf',
            onModelLoaded,
            onProgress,
            onLoadError);
```

```js
function onModelLoaded(gltf) {
	console.log('Model loaded', gltf);
	gltf.scene;
	scene.add(gltf.scene);
}
```

```js
function onProgress(event) {
	console.log((event.loaded / event.total) * 100 + '% loaded');
}
```

* CallBacks vs. Promises

## Formate USDZ

Formato alternativo adoptado por Apple y desarrollado en conjunto por Pixar

Object promise

## Eventos del Teclado

Primero hay definir un callback para cuando se produzca un evento sobre un
objeto html en particular mientras la ventana del navegador este en foco

```js
Document.addEventListener('keydown', keyHandler)
```

```js
function keyHandler(event) {
    if (event.key == 'c' && event.ctrlKey && event.shiftKey) {
        console.log("Ctrl+Shift+c");
    }
}
```

### Controles

#### FlyControl

#### OrbitControl

#### TrackballControl

### User Interface

#### uil

[uil](https://github.com/lo-th/uil)

### dat GUI

[dat.GUI](https://github.com/dataarts/dat.gui)

```js
let gui = new dat.GUI();
gui.add(params, 'cantidadTotal', 0, 10).step(1);
```

## Matriz de Normales

* El comportamiento deseado al trasladar y escalar un objeto, es que las
  normales conserven su dirección
* El comportamiento deseado al rotar un objeto es que las normales cambien su
  dirección
* Casos particulares incluyen escalado con diferente magnitud en sus componentes

## Shaders

### Uniforms

Las `uniforms` son variables globales que comparten todas las unidades de
procesamiento de la GPU

```js
// Se definen los uniforms que se usarán en los shaders
uniforms: {
    modelMatrix: { value: new THREE.Matrix4() },
    viewMatrix: { value: new THREE.Matrix4() },
    projectionMatrix: { value: new THREE.Matrix4() },
},
```

### Varyings

Las `varyings` son variables globales que el Vertex Shader comparte con el
Fragment Shader

### Vertex Shader

```c
precision highp float;

attribute vec3 position;
attribute vec2 uv;

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;
varying vec2 vUv;

void main() {
    vec3 pos = position;	

    gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(pos, 1.0);
    vUv = uv;
}
```

### Fragment Shader

```c
precision highp float;
varying vec2 vUv;

void main() {
    // Se pinta el fragmento con las coordenadas de textura
    gl_FragColor = vec4(vUv, 0.0, 1.0);
}
```

#### Normal a una Superficie deformada

* Versión de WebGl 2.0 (OpenGL 3.0) proporciona el calculo de las derivadas
* Se calcula el vector gradiente

```c
vec3 x=dFdx(vViewPos);
vec3 y=dFdy(vViewPos);
```

```c
// Normal de la superficie surge de la normalización del producto cruz de los
// vectores gradiente
vec3 normal=normalize(cross(x,y));
```
